@@ -41,6 +41,8 @@
 #include "netk-xutils.h"
 #include "netk-private.h"
 
+#include "xfce-gdk-extensions.h"
+
 #define NETK_TYPE_TASK              (netk_task_get_type ())
 #define NETK_TASK(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), NETK_TYPE_TASK, NetkTask))
 #define NETK_TASK_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), NETK_TYPE_TASK, NetkTaskClass))
@@ -48,6 +50,8 @@
 #define NETK_IS_TASK_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), NETK_TYPE_TASK))
 #define NETK_TASK_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), NETK_TYPE_TASK, NetkTaskClass))
 
+#define NETK_TASKLIST_GET_PRIVATE(o)    (G_TYPE_INSTANCE_GET_PRIVATE ((o), NETK_TYPE_TASKLIST, NetkTasklistPrivate))
+
 typedef struct _NetkTask NetkTask;
 typedef struct _NetkTaskClass NetkTaskClass;
 
@@ -68,6 +72,8 @@ typedef struct _NetkTaskClass NetkTaskClass;
 #define NETK_ELLIPSIZE_END 3
 #endif
 
+#define MAX_URGENT_BLINK_COUNT 18
+
 typedef enum
 {
     NETK_TASK_CLASS_GROUP,
@@ -113,6 +119,7 @@ struct _NetkTask
     guint button_activate;
 
     guint blink_timeout_id;
+    gint blink_count;
 };
 
 struct _NetkTaskClass
@@ -146,7 +153,6 @@ struct _NetkTasklistPrivate
     NetkTasklistGroupingType grouping;
     gint grouping_limit;
 
-    guint activate_timeout_id;
     guint screen_connections[N_SCREEN_CONNECTIONS];
     guint idle_callback_tag;
     
@@ -161,6 +167,7 @@ struct _NetkTasklistPrivate
     GDestroyNotify free_icon_loader_data;
 
     gboolean show_label;
+    GtkReliefStyle relief;
 };
 
 
@@ -173,7 +180,10 @@ static NetkTask *netk_task_new_from_window (NetkTasklist * tasklist,
 static NetkTask *netk_task_new_from_class_group (NetkTasklist * tasklist,
                                                  NetkClassGroup * class_group);
 
+static gboolean netk_task_get_demands_attention (NetkTask * task);
 static gboolean  urgent_timeout_callback(gpointer data);
+static void  queue_urgent_timeout(gpointer data);
+static void  unqueue_urgent_timeout(gpointer data);
 static char *netk_task_get_text (NetkTask * task);
 static GdkPixbuf *netk_task_get_icon (NetkTask * task);
 static gint netk_task_compare (gconstpointer a, gconstpointer b);
@@ -218,9 +228,7 @@ static void netk_tasklist_connect_window (NetkTasklist * tasklist,
 
 static void netk_tasklist_change_active_task (NetkTasklist * tasklist,
                                               NetkTask * active_task);
-#if 0  /* unused? */
-static gboolean netk_tasklist_change_active_timeout (gpointer data);
-#endif
+
 static void netk_tasklist_activate_task_window (NetkTask * task);
 
 static void netk_tasklist_update_icon_geometries (NetkTasklist * tasklist);
@@ -228,41 +236,32 @@ static void netk_tasklist_connect_screen (NetkTasklist * tasklist,
                                           NetkScreen * screen);
 static void netk_tasklist_disconnect_screen (NetkTasklist * tasklist);
 
-static gpointer task_parent_class;
-static gpointer tasklist_parent_class;
+static GObjectClass *task_parent_class;
+static GObjectClass *tasklist_parent_class;
 
 GType
 netk_task_get_type (void)
 {
-    static GType object_type = 0;
+    static GType type = G_TYPE_INVALID;
 
-    g_type_init ();
-
-    if (!object_type)
+    if (G_UNLIKELY (type == G_TYPE_INVALID))
     {
-        static const GTypeInfo object_info = {
-            sizeof (NetkTaskClass),
-            (GBaseInitFunc) NULL,
-            (GBaseFinalizeFunc) NULL,
-            (GClassInitFunc) netk_task_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-            sizeof (NetkTask),
-            0,                  /* n_preallocs */
-            (GInstanceInitFunc) netk_task_init,
-        };
-
-        object_type =
-            g_type_register_static (G_TYPE_OBJECT, "NetkTask", &object_info,
-                                    0);
+        type = _netk_g_type_register_simple (G_TYPE_OBJECT,
+                                             "NetkTask",
+                                             sizeof (NetkTaskClass),
+                                             netk_task_class_init,
+                                             sizeof (NetkTask),
+                                             netk_task_init,
+                                             0);
     }
 
-    return object_type;
+    return type;
 }
 
 static void
 netk_task_init (NetkTask * task)
 {
+    
 }
 
 static void
@@ -343,25 +342,25 @@ netk_task_finalize (GObject * object)
 
     if (task->menu)
     {
-        g_object_unref (task->menu);
+        gtk_widget_destroy (task->menu);
         task->menu = NULL;
     }
 
     if (task->action_menu)
     {
-      g_object_unref (task->action_menu);
+      g_object_unref (G_OBJECT (task->action_menu));
       task->action_menu = NULL;
     }
 
     if (task->window)
     {
-        g_object_unref (task->window);
+        g_object_unref (G_OBJECT (task->window));
         task->window = NULL;
     }
 
     if (task->class_group)
     {
-        g_object_unref (task->class_group);
+        g_object_unref (G_OBJECT (task->class_group));
         task->class_group = NULL;
     }
 
@@ -377,30 +376,20 @@ netk_task_finalize (GObject * object)
 GType
 netk_tasklist_get_type (void)
 {
-    static GType object_type = 0;
+    static GType type = G_TYPE_INVALID;
 
-    g_type_init ();
-
-    if (!object_type)
+    if (G_UNLIKELY (type == G_TYPE_INVALID))
     {
-        static const GTypeInfo object_info = {
-            sizeof (NetkTasklistClass),
-            (GBaseInitFunc) NULL,
-            (GBaseFinalizeFunc) NULL,
-            (GClassInitFunc) netk_tasklist_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-            sizeof (NetkTasklist),
-            0,                  /* n_preallocs */
-            (GInstanceInitFunc) netk_tasklist_init,
-        };
-
-        object_type =
-            g_type_register_static (GTK_TYPE_CONTAINER, "NetkTasklist",
-                                    &object_info, 0);
+        type = _netk_g_type_register_simple (GTK_TYPE_CONTAINER,
+                                             "NetkTasklist",
+                                             sizeof (NetkTasklistClass),
+                                             netk_tasklist_class_init,
+                                             sizeof (NetkTasklist),
+                                             netk_tasklist_init,
+                                             0);
     }
 
-    return object_type;
+    return type;
 }
 
 static void
@@ -408,8 +397,8 @@ netk_tasklist_init (NetkTasklist * tasklist)
 {
     GTK_WIDGET_SET_FLAGS (GTK_WIDGET (tasklist), GTK_NO_WINDOW);
 
-    tasklist->priv = g_new0 (NetkTasklistPrivate, 1);
-
+    tasklist->priv = NETK_TASKLIST_GET_PRIVATE (tasklist);
+    
     tasklist->priv->include_all_workspaces = FALSE;
 
     tasklist->priv->win_hash = g_hash_table_new (NULL, NULL);
@@ -431,6 +420,8 @@ netk_tasklist_class_init (NetkTasklistClass * klass)
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
     GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
 
+    g_type_class_add_private (klass, sizeof (NetkTasklistPrivate));
+
     tasklist_parent_class = g_type_class_peek_parent (klass);
 
     object_class->finalize = netk_tasklist_finalize;
@@ -470,11 +461,6 @@ netk_tasklist_finalize (GObject * object)
     g_hash_table_destroy (tasklist->priv->class_group_hash);
     tasklist->priv->class_group_hash = NULL;
 
-    if (tasklist->priv->activate_timeout_id != 0)
-    {
-        g_source_remove (tasklist->priv->activate_timeout_id);
-    }
-
     if (tasklist->priv->idle_callback_tag != 0)
     {
         g_source_remove (tasklist->priv->idle_callback_tag);
@@ -482,7 +468,7 @@ netk_tasklist_finalize (GObject * object)
 
     if (tasklist->priv->tooltips)
     {
-        g_object_unref (tasklist->priv->tooltips);
+        g_object_unref (G_OBJECT (tasklist->priv->tooltips));
         tasklist->priv->tooltips = NULL;
     }
 
@@ -490,9 +476,6 @@ netk_tasklist_finalize (GObject * object)
     tasklist->priv->size_hints = NULL;
     tasklist->priv->size_hints_len = 0;
 
-    g_free (tasklist->priv);
-    tasklist->priv = NULL;
-
     G_OBJECT_CLASS (tasklist_parent_class)->finalize (object);
 }
 
@@ -510,6 +493,20 @@ netk_tasklist_set_show_label (NetkTasklist * tasklist,
     netk_tasklist_update_lists (tasklist);
 }
 
+void 
+netk_tasklist_set_button_relief (NetkTasklist * tasklist, 
+                                 GtkReliefStyle relief)
+{
+    g_return_if_fail (NETK_IS_TASKLIST (tasklist));
+
+    if (tasklist->priv->relief == relief)
+        return;
+
+    tasklist->priv->relief = relief;
+    netk_tasklist_update_lists (tasklist);
+    gtk_widget_queue_resize (GTK_WIDGET (tasklist));
+}
+
 void
 netk_tasklist_set_grouping (NetkTasklist * tasklist,
                             NetkTasklistGroupingType grouping)
@@ -890,6 +887,8 @@ netk_tasklist_size_request (GtkWidget * widget, GtkRequisition * requisition)
         }
     }
 
+    g_list_free (ungrouped_class_groups);
+
     /* Always let you go down to a zero size: */
     if (array->len > 0)
         g_array_index (array, int, array->len - 1) = 0;
@@ -1038,11 +1037,13 @@ netk_tasklist_size_allocate (GtkWidget * widget, GtkAllocation * allocation)
         i++;
         l = l->next;
     }
-    g_list_free (visible_tasks);
 
     /* Update icon geometries. */
     netk_tasklist_update_icon_geometries (tasklist);
 
+    g_list_free (visible_tasks);
+    g_list_free (ungrouped_class_groups);
+
     GTK_WIDGET_CLASS (tasklist_parent_class)->size_allocate (widget,
                                                              allocation);
 }
@@ -1119,7 +1120,7 @@ netk_tasklist_remove (GtkContainer * container, GtkWidget * widget)
                 g_list_remove (tasklist->priv->windows, task);
 
             gtk_widget_unparent (widget);
-            g_object_unref (task);
+            g_object_unref (G_OBJECT (task));
             break;
         }
     }
@@ -1137,7 +1138,7 @@ netk_tasklist_remove (GtkContainer * container, GtkWidget * widget)
                 g_list_remove (tasklist->priv->class_groups, task);
 
             gtk_widget_unparent (widget);
-            g_object_unref (task);
+            g_object_unref (G_OBJECT (task));
             break;
         }
     }
@@ -1238,6 +1239,7 @@ netk_tasklist_new (NetkScreen * screen)
     netk_tasklist_set_screen (tasklist, screen);
 
     tasklist->priv->show_label=TRUE;
+    tasklist->priv->relief=GTK_RELIEF_NORMAL;
 
     return GTK_WIDGET (tasklist);
 }
@@ -1498,7 +1500,7 @@ netk_tasklist_active_window_changed (NetkScreen * screen,
 {
     NetkWindow *active_window;
     NetkTask *active_task;
-
+    
     active_window = netk_screen_get_active_window (screen);
 
     active_task =
@@ -1656,20 +1658,6 @@ netk_task_position_menu (GtkMenu * menu, gint * x, gint * y,
     *push_in = TRUE;
 }
 
-#if 0 /* unused? */
-static gboolean
-netk_tasklist_change_active_timeout (gpointer data)
-{
-    NetkTasklist *tasklist = NETK_TASKLIST (data);
-
-    tasklist->priv->activate_timeout_id = 0;
-
-    netk_tasklist_active_window_changed (tasklist->priv->screen, tasklist);
-
-    return FALSE;
-}
-#endif
-
 static void
 netk_task_menu_activated (GtkMenuItem * menu_item, gpointer data)
 {
@@ -1727,11 +1715,6 @@ netk_tasklist_activate_task_window (NetkTask * task)
         }
     }
 
-    if (tasklist->priv->activate_timeout_id)
-    {
-        g_source_remove (tasklist->priv->activate_timeout_id);
-    }
-    
     netk_tasklist_change_active_task (tasklist, task);
 }
 
@@ -1772,6 +1755,8 @@ netk_task_popup_menu (NetkTask * task, gboolean action_submenu)
 
         text = netk_task_get_text (win_task);
         menu_item = gtk_image_menu_item_new_with_label (text);
+        if (netk_task_get_demands_attention (win_task))
+            p_make_gtk_label_bold (GTK_LABEL (GTK_BIN (menu_item)->child));
         g_free (text);
 
         pixbuf = netk_task_get_icon (win_task);
@@ -1781,7 +1766,7 @@ netk_task_popup_menu (NetkTask * task, gboolean action_submenu)
             gtk_widget_show (image);
             gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
                                            image);
-            g_object_unref (pixbuf);
+            g_object_unref (G_OBJECT (pixbuf));
         }
 
         gtk_widget_show (menu_item);
@@ -1799,6 +1784,7 @@ netk_task_popup_menu (NetkTask * task, gboolean action_submenu)
         l = l->next;
     }
 
+    gtk_menu_set_screen (GTK_MENU (menu), xfce_gdk_display_locate_monitor_with_pointer (NULL, NULL));
     gtk_widget_show (menu);
     gtk_menu_popup (GTK_MENU (menu), NULL, NULL, netk_task_position_menu,
                     task->button, 1, gtk_get_current_event_time ());
@@ -1938,7 +1924,7 @@ netk_task_scale_icon (GdkPixbuf * orig, gboolean minimized)
     }
 
     if (orig == pixbuf)
-        g_object_ref (pixbuf);
+        g_object_ref (G_OBJECT (pixbuf));
 
     return pixbuf;
 }
@@ -1974,27 +1960,76 @@ netk_task_get_icon (NetkTask * task)
     return pixbuf;
 }
 
+static gboolean
+netk_task_get_demands_attention (NetkTask * task)
+{
+    GList *l;
+    NetkTask *win_task;
+    gboolean demands_attention = FALSE;
+
+    switch (task->type)
+    {
+        case NETK_TASK_CLASS_GROUP:
+            l = task->windows;
+            while (l)
+            {
+                win_task = NETK_TASK (l->data);
+
+                if (netk_window_or_transient_demands_attention
+                    (win_task->window))
+                {
+                    demands_attention = TRUE;
+                    break;
+                }
+
+                l = l->next;
+            }
+            break;
+
+        case NETK_TASK_WINDOW:
+            demands_attention =
+                netk_window_or_transient_demands_attention (task->window);
+            break;
+    }
+
+    return demands_attention != FALSE;
+}
 
 static void
 netk_task_update_visible_state (NetkTask * task)
 {
     GdkPixbuf *pixbuf;
     char *text;
+    gboolean urgent = netk_task_get_demands_attention (task);
 
     pixbuf = netk_task_get_icon (task);
     gtk_image_set_from_pixbuf (GTK_IMAGE (task->image), pixbuf);
     if (pixbuf)
-        g_object_unref (pixbuf);
-
+        g_object_unref (G_OBJECT (pixbuf));
+    
     text = netk_task_get_text (task);
     if (text != NULL)
     {
-        if (task->tasklist->priv->show_label)
+        if (task->tasklist->priv->show_label && task->label)
+        {
             gtk_label_set_text (GTK_LABEL (task->label), text);
+            if (urgent)
+            {
+                p_make_gtk_label_bold ((GTK_LABEL (task->label)));
+            }
+            else
+            {
+                p_make_gtk_label_normal ((GTK_LABEL (task->label)));
+            }
+        }
         gtk_tooltips_set_tip (task->tasklist->priv->tooltips, task->button,
                               text, NULL);
         g_free (text);
     }
+    if (urgent)
+        queue_urgent_timeout(task);
+    else
+        unqueue_urgent_timeout(task);
 
     gtk_widget_queue_resize (GTK_WIDGET (task->tasklist));
 }
@@ -2004,7 +2039,7 @@ netk_task_state_changed (NetkWindow * window, NetkWindowState changed_mask,
                          NetkWindowState new_state, gpointer data)
 {
     NetkTasklist *tasklist = NETK_TASKLIST (data);
-
+    
     if (changed_mask & NETK_WINDOW_STATE_SKIP_TASKLIST)
     {
         netk_tasklist_update_lists (tasklist);
@@ -2012,58 +2047,27 @@ netk_task_state_changed (NetkWindow * window, NetkWindowState changed_mask,
         return;
     }
 
-    if (changed_mask & NETK_WINDOW_STATE_MINIMIZED)
+    if (changed_mask & NETK_WINDOW_STATE_MINIMIZED              ||
+        changed_mask & NETK_WINDOW_STATE_URGENT                 || 
+        changed_mask & NETK_WINDOW_STATE_DEMANDS_ATTENTION      )
     {
-        NetkTask *win_task;
+        NetkTask *win_task = NULL, *class_group_task = NULL;
 
         win_task = g_hash_table_lookup (tasklist->priv->win_hash, window);
-        if (win_task)
-        {
-            NetkTask *class_group_task;
-            
-            netk_task_update_visible_state (win_task);
-
-            class_group_task =
-                g_hash_table_lookup (tasklist->priv->class_group_hash,
-                                     win_task->class_group);
-
-            if (class_group_task)
-                netk_task_update_visible_state (class_group_task);
-        }
-    }
     
-    if (changed_mask & NETK_WINDOW_STATE_URGENT)
-    {
-        NetkTask *win_task;
-
-        win_task = g_hash_table_lookup (tasklist->priv->win_hash, window);
         if (win_task)
         {
-            if (win_task->blink_timeout_id)
-            {
-                g_source_remove (win_task->blink_timeout_id);
-            }
-
-            if(new_state & NETK_WINDOW_STATE_URGENT)
-            {
-                win_task->blink_timeout_id = 
-                    g_timeout_add (500, urgent_timeout_callback, win_task);
-            }
-            else 
-            {
-                GtkRcStyle *style; 
-
-                style = gtk_widget_get_modifier_style (win_task->button);
-                style->color_flags[GTK_STATE_NORMAL] = 0;
-                gtk_widget_modify_style (win_task->button, style);
-                if(win_task->tasklist->priv->show_label)
-                {
-                    style = gtk_widget_get_modifier_style (win_task->label);
-                    style->color_flags[GTK_STATE_NORMAL] = 0;
-                    gtk_widget_modify_style (win_task->label, style);
-                }
-            }
+            class_group_task = 
+                g_hash_table_lookup (tasklist->priv->class_group_hash, 
+                                     win_task->class_group);
         }
+
+        if (class_group_task)
+        {
+            netk_task_update_visible_state (class_group_task);
+        }
+
+        netk_task_update_visible_state (win_task);
     }
 }
 
@@ -2183,8 +2187,8 @@ netk_task_button_press_event (GtkWidget * widget,
               g_object_add_weak_pointer (G_OBJECT (task->action_menu),
                                          (void**) &task->action_menu);
 
-              //gtk_menu_set_screen (GTK_MENU (task->action_menu),
-              //                     _netk_screen_get_gdk_screen (task->tasklist->priv->screen));
+	      gtk_menu_set_screen (GTK_MENU (task->action_menu), 
+				   xfce_gdk_display_locate_monitor_with_pointer (NULL, NULL));
 
               gtk_widget_show (task->action_menu);
               gtk_menu_popup (GTK_MENU (task->action_menu),
@@ -2214,6 +2218,8 @@ netk_task_create_widgets (NetkTask * task)
 
     task->button = gtk_toggle_button_new ();
     task->button_activate = 0;
+    gtk_button_set_relief (GTK_BUTTON (task->button), task->tasklist->priv->relief);
+
     g_object_add_weak_pointer (G_OBJECT (task->button),
                                (void **) &task->button);
 
@@ -2227,7 +2233,7 @@ netk_task_create_widgets (NetkTask * task)
     if (pixbuf)
     {
         task->image = gtk_image_new_from_pixbuf (pixbuf);
-        g_object_unref (pixbuf);
+        g_object_unref (G_OBJECT (pixbuf));
     }
     else
     {
@@ -2242,15 +2248,18 @@ netk_task_create_widgets (NetkTask * task)
                       task->image, 0, 1, 0, 1, 0, GTK_EXPAND, 0, 0);
 
     if (task->tasklist->priv->show_label) {
+        gtk_table_set_col_spacings (GTK_TABLE (table), 4);
         task->label = gtk_label_new (text);
         gtk_misc_set_alignment (GTK_MISC (task->label), 0.5, 0.5);
-        if ((gtk_major_version == 2 && gtk_minor_version >= 6) || gtk_major_version > 2)
-        {
-            g_object_set (G_OBJECT (task->label), "ellipsize", NETK_ELLIPSIZE_END, NULL);
-        }
+        gtk_label_set_ellipsize (GTK_LABEL (task->label), PANGO_ELLIPSIZE_END);
         gtk_widget_show (task->label);
-        gtk_table_attach (GTK_TABLE (table),
-                      task->label, 1, 2, 0, 1, GTK_EXPAND|GTK_FILL, GTK_EXPAND, 0, 0);
+        gtk_table_attach (GTK_TABLE (table), task->label, 1, 2, 0, 1, 
+                          GTK_EXPAND|GTK_FILL|GTK_SHRINK, GTK_EXPAND, 0, 0);
+    }
+    else
+    {
+        gtk_table_set_col_spacings (GTK_TABLE (table), 0);
+        task->label = NULL;
     }
 
     gtk_container_add (GTK_CONTAINER (task->button), table);
@@ -2317,30 +2326,40 @@ urgent_timeout_callback (gpointer data)
     GtkStyle *style;
     GtkRcStyle *mod;
     GdkColor c;
-    NetkWindowState state;
 
     g_return_val_if_fail (task, FALSE);
 
-    state = netk_window_get_state (task->window);
-
-    if(state & NETK_WINDOW_STATE_URGENT)
+    if(netk_task_get_demands_attention (task))
     {
+        if(task->blink_count > MAX_URGENT_BLINK_COUNT)
+        {
+            return TRUE;
+        }
+        else
+        {
+            ++task->blink_count;
+        }
+        
         style = gtk_widget_get_style (task->button);
         mod = gtk_widget_get_modifier_style (task->button);
         c = style->bg[GTK_STATE_SELECTED];
         if(mod->color_flags[GTK_STATE_NORMAL] & GTK_RC_BG)
         {
+            gtk_button_set_relief (GTK_BUTTON (task->button), 
+                                   task->tasklist->priv->relief);
             mod->color_flags[GTK_STATE_NORMAL] &= ~(GTK_RC_BG);
             gtk_widget_modify_style(task->button, mod);
         }
         else
         {
+            gtk_button_set_relief (GTK_BUTTON (task->button), 
+                                   GTK_RELIEF_NORMAL);
             mod->color_flags[GTK_STATE_NORMAL] |= GTK_RC_BG;
             mod->bg[GTK_STATE_NORMAL] = c;
             gtk_widget_modify_style(task->button, mod);
         }
 
-        if(task->tasklist->priv->show_label)
+        if(task->tasklist->priv->show_label && task->label)
         {
             style = gtk_widget_get_style (task->label);
             mod = gtk_widget_get_modifier_style (task->label);
@@ -2357,12 +2376,58 @@ urgent_timeout_callback (gpointer data)
                 mod->fg[GTK_STATE_NORMAL] = c;
                 gtk_widget_modify_style(task->label, mod);
             }
+
+            p_make_gtk_label_bold (GTK_LABEL (task->label));
         }
     }
+    else if(task->tasklist->priv->show_label && task->label)
+    {
+        unqueue_urgent_timeout (task);
+    }
 
-    return (state & NETK_WINDOW_STATE_URGENT);
+    return (netk_task_get_demands_attention (task));
 }
  
+static void  
+queue_urgent_timeout(gpointer data)
+{
+    NetkTask *task = data;
+
+    if (task->blink_timeout_id == 0)
+    {
+        task->blink_timeout_id = 
+                    g_timeout_add (500, urgent_timeout_callback, task);
+    }
+}
+
+static void  
+unqueue_urgent_timeout(gpointer data)
+{
+    NetkTask *task = data;
+
+    if (task->blink_timeout_id > 0)
+    {
+        GtkRcStyle *style; 
+
+        g_source_remove (task->blink_timeout_id);
+
+        gtk_button_set_relief (GTK_BUTTON (task->button), 
+                               task->tasklist->priv->relief);
+        style = gtk_widget_get_modifier_style (task->button);
+        style->color_flags[GTK_STATE_NORMAL] = 0;
+        gtk_widget_modify_style (task->button, style);
+        if(task->tasklist->priv->show_label && task->label)
+        {
+            style = gtk_widget_get_modifier_style (task->label);
+            style->color_flags[GTK_STATE_NORMAL] = 0;
+            gtk_widget_modify_style (task->label, style);
+            p_make_gtk_label_normal (GTK_LABEL (task->label));
+        }
+    }
+    task->blink_timeout_id = 0;
+    task->blink_count = 0;
+}
+
 static void
 draw_dot (GdkWindow * window, GdkGC * lgc, GdkGC * dgc, int x, int y)
 {
@@ -2468,16 +2533,16 @@ netk_task_new_from_window (NetkTasklist * tasklist, NetkWindow * window)
     task = g_object_new (NETK_TYPE_TASK, NULL);
 
     task->type = NETK_TASK_WINDOW;
-    task->window = g_object_ref (window);
-    task->class_group = g_object_ref (netk_window_get_class_group (window));
+    task->window = g_object_ref (G_OBJECT (window));
+    task->class_group = g_object_ref (G_OBJECT (netk_window_get_class_group (window)));
     task->tasklist = tasklist;
     task->blink_timeout_id = 0;
 
     netk_task_create_widgets (task);
     
-    if (netk_window_get_state (window) & NETK_WINDOW_STATE_URGENT)
+    if (netk_task_get_demands_attention (task))
     {
-        task->blink_timeout_id = g_timeout_add (500, urgent_timeout_callback, task);
+        queue_urgent_timeout (task);
     }
 
     return task;
@@ -2492,9 +2557,10 @@ netk_task_new_from_class_group (NetkTasklist * tasklist,
     task = g_object_new (NETK_TYPE_TASK, NULL);
 
     task->type = NETK_TASK_CLASS_GROUP;
-    task->class_group = g_object_ref (class_group);
+    task->class_group = g_object_ref (G_OBJECT (class_group));
     task->window = NULL;
     task->tasklist = tasklist;
+    task->blink_timeout_id = 0;
 
     netk_task_create_widgets (task);
 
@@ -2502,5 +2568,67 @@ netk_task_new_from_class_group (NetkTasklist * tasklist,
                              G_CALLBACK (netk_task_class_group_expose),
                              G_OBJECT (task), G_CONNECT_AFTER);
 
+    if (netk_task_get_demands_attention (task))
+    {
+        queue_urgent_timeout (task);
+    }
+
     return task;
 }
+
+NetkScreen*
+netk_tasklist_get_screen(NetkTasklist * tasklist)
+{
+    g_return_val_if_fail (NETK_IS_TASKLIST (tasklist), NULL);
+
+    return tasklist->priv->screen;
+}
+
+NetkTasklistGroupingType
+netk_tasklist_get_grouping(NetkTasklist * tasklist)
+{
+    g_return_val_if_fail (NETK_IS_TASKLIST (tasklist), NETK_TASKLIST_NEVER_GROUP);
+
+    return tasklist->priv->grouping;
+}
+
+gboolean 
+netk_tasklist_get_switch_workspace_on_unminimize(NetkTasklist * tasklist)
+{
+    g_return_val_if_fail (NETK_IS_TASKLIST (tasklist), FALSE);
+
+    return tasklist->priv->switch_workspace_on_unminimize;
+}
+
+gint 
+netk_tasklist_get_grouping_limit(NetkTasklist * tasklist)
+{
+    g_return_val_if_fail (NETK_IS_TASKLIST (tasklist), 0);
+
+    return tasklist->priv->grouping_limit;
+}
+
+gboolean 
+netk_tasklist_get_include_all_workspaces(NetkTasklist * tasklist)
+{
+    g_return_val_if_fail (NETK_IS_TASKLIST (tasklist), FALSE);
+
+    return tasklist->priv->include_all_workspaces;
+}
+
+gboolean 
+netk_tasklist_get_show_label(NetkTasklist * tasklist)
+{
+    g_return_val_if_fail (NETK_IS_TASKLIST (tasklist), FALSE);
+
+    return tasklist->priv->show_label;
+}
+
+GtkReliefStyle 
+netk_tasklist_get_button_relief(NetkTasklist * tasklist)
+{
+    g_return_val_if_fail (NETK_IS_TASKLIST (tasklist), GTK_RELIEF_NONE);
+    
+    return tasklist->priv->relief;
+}
+
