@@ -1,26 +1,28 @@
-/*      $Id: misc.c,v 1.50 2004/10/20 20:49:33 olivier Exp $
- 
+/*      $Id: misc.c 26999 2008-05-31 14:35:42Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
+
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
- 
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
+
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <X11/Xlib.h>
@@ -34,7 +36,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 
 #include "display.h"
 #include "screen.h"
@@ -42,32 +44,35 @@
 #include "client.h"
 #include "misc.h"
 
-void
+unsigned int
 getMouseXY (ScreenInfo *screen_info, Window w, int *x2, int *y2)
 {
     Window w1, w2;
-    gint x1, y1, m;
+    unsigned int mask;
+    int x1, y1;
 
     TRACE ("entering getMouseXY");
 
-    XQueryPointer (myScreenGetXDisplay (screen_info), w, &w1, &w2, &x1, &y1, x2, y2, &m);
+    XQueryPointer (myScreenGetXDisplay (screen_info), w, &w1, &w2, &x1, &y1, x2, y2, &mask);
+    return mask;
 }
 
 Window
 getMouseWindow (ScreenInfo *screen_info, Window w)
 {
     Window w1, w2;
-    int x1, y1, x2, y2, m;
+    unsigned int mask;
+    int x1, y1, x2, y2;
 
     TRACE ("entering getMouseWindow");
 
-    XQueryPointer (myScreenGetXDisplay (screen_info), w, &w1, &w2, &x1, &y1, &x2, &y2, &m);
+    XQueryPointer (myScreenGetXDisplay (screen_info), w, &w1, &w2, &x1, &y1, &x2, &y2, &mask);
     return w2;
 }
 
 GC
 createGC (ScreenInfo *screen_info, char *col, int func, XFontStruct * font,
-    int line_width, gboolean inc_sw)
+          int line_width, gboolean inc_sw)
 {
     XGCValues gv;
     XColor xc1, xc2;
@@ -101,38 +106,64 @@ createGC (ScreenInfo *screen_info, char *col, int func, XFontStruct * font,
 }
 
 void
-sendClientMessage (ScreenInfo *screen_info, Window w, Atom a, Atom x, Time timestamp)
+sendClientMessage (ScreenInfo *screen_info, Window w, int atom_id, Time timestamp)
 {
+    DisplayInfo *display_info;
     XClientMessageEvent ev;
 
+    g_return_if_fail ((atom_id > 0) && (atom_id < ATOM_COUNT));
     TRACE ("entering sendClientMessage");
 
+    display_info = screen_info->display_info;
     ev.type = ClientMessage;
     ev.window = w;
-    ev.message_type = a;
+    ev.message_type = display_info->atoms[WM_PROTOCOLS];
     ev.format = 32;
-    ev.data.l[0] = x;
+    ev.send_event = TRUE;
+    ev.data.l[0] = display_info->atoms[atom_id];
     ev.data.l[1] = timestamp;
     XSendEvent (myScreenGetXDisplay (screen_info), w, FALSE, 0L, (XEvent *)&ev);
 }
 
+void
+sendRootMessage (ScreenInfo *screen_info, int atom_id, long value, Time timestamp)
+{
+    DisplayInfo *display_info;
+    XClientMessageEvent ev;
+
+    g_return_if_fail ((atom_id > 0) && (atom_id < ATOM_COUNT));
+    TRACE ("entering sendClientMessage");
+
+    display_info = screen_info->display_info;
+    ev.type = ClientMessage;
+    ev.window = screen_info->xroot;
+    ev.message_type = display_info->atoms[atom_id];
+    ev.format = 32;
+    ev.data.l[0] = value;
+    ev.data.l[1] = timestamp;
+    XSendEvent (myScreenGetXDisplay (screen_info), screen_info->xroot, FALSE,
+                SubstructureRedirectMask | SubstructureNotifyMask, (XEvent *)&ev);
+}
+
 /*
  * it's safer to grab the display before calling this routine
- * Returns true if the given window is present and mapped on root 
+ * Returns true if the given window is present and mapped on root
  */
 gboolean
 checkWindowOnRoot(ScreenInfo *screen_info, Window w)
 {
-    DisplayInfo *display_info = NULL;
+    DisplayInfo *display_info;
     Window dummy_root, parent;
-    Window *wins = NULL;
-    unsigned int count;
+    Window *wins;
     Status test;
-    
+    unsigned int count;
+
     g_return_val_if_fail (screen_info != NULL, FALSE);
     g_return_val_if_fail (w != None, FALSE);
 
     display_info = screen_info->display_info;
+    wins = NULL;
+
     gdk_error_trap_push ();
     test = XQueryTree(display_info->dpy, w, &dummy_root, &parent, &wins, &count);
     if (wins)
@@ -145,61 +176,77 @@ checkWindowOnRoot(ScreenInfo *screen_info, Window w)
 void
 placeSidewalks(ScreenInfo *screen_info, gboolean activate)
 {
+    NetWmDesktopLayout l;
+
     g_return_if_fail (MYWINDOW_XWINDOW (screen_info->sidewalk[0]) != None);
     g_return_if_fail (MYWINDOW_XWINDOW (screen_info->sidewalk[1]) != None);
     g_return_if_fail (MYWINDOW_XWINDOW (screen_info->sidewalk[2]) != None);
     g_return_if_fail (MYWINDOW_XWINDOW (screen_info->sidewalk[3]) != None);
 
-    if (activate)
+    l = screen_info->desktop_layout;
+    if ((activate) && (l.cols > 1))
     {
-        xfwmWindowShow (&screen_info->sidewalk[0], 
+        /*left*/
+        xfwmWindowShow (&screen_info->sidewalk[0],
                         0, 0,
-                        1, gdk_screen_get_height (screen_info->gscr), FALSE);
-        xfwmWindowShow (&screen_info->sidewalk[1],
-                        gdk_screen_get_width (screen_info->gscr) - 1, 0, 
-                        1, gdk_screen_get_height (screen_info->gscr), FALSE);
-        xfwmWindowShow (&screen_info->sidewalk[2], 
-                        0, 0, 
-                        gdk_screen_get_width (screen_info->gscr), 1, FALSE);
-        xfwmWindowShow (&screen_info->sidewalk[3],
-                        0, gdk_screen_get_height (screen_info->gscr) - 1, 
-                        gdk_screen_get_width (screen_info->gscr), 1, FALSE);
+                        1, screen_info->height, FALSE);
+
+        /*right*/
+         xfwmWindowShow (&screen_info->sidewalk[1],
+                        screen_info->width - 1, 0,
+                        1, screen_info->height, FALSE);
     }
-    else
+    else /* Place the windows off screen */
     {
-        /* Place the windows off screen */
-        xfwmWindowShow (&screen_info->sidewalk[0], 
+        /*left*/
+        xfwmWindowShow (&screen_info->sidewalk[0],
                         -1, 0,
-                        1, gdk_screen_get_height (screen_info->gscr), FALSE);
+                        1, screen_info->height, FALSE);
+
+        /*right*/
         xfwmWindowShow (&screen_info->sidewalk[1],
-                        gdk_screen_get_width (screen_info->gscr), 0, 
-                        1, gdk_screen_get_height (screen_info->gscr), FALSE);
-        xfwmWindowShow (&screen_info->sidewalk[2], 
-                        0, -1, 
-                        gdk_screen_get_width (screen_info->gscr), 1, FALSE);
+                        screen_info->width, 0,
+                        1, screen_info->height, FALSE);
+    }
+
+    if ((activate) && (l.rows > 1))
+    {
+        /*top*/
+        xfwmWindowShow (&screen_info->sidewalk[2],
+                        0, 0,
+                        screen_info->width, 1, FALSE);
+
+        /*bottom*/
         xfwmWindowShow (&screen_info->sidewalk[3],
-                        0, gdk_screen_get_height (screen_info->gscr), 
-                        gdk_screen_get_width (screen_info->gscr), 1, FALSE);
+                        0, screen_info->height - 1,
+                        screen_info->width, 1, FALSE);
+    }
+    else /* Place the windows off screen */
+    {
+        /*top*/
+        xfwmWindowShow (&screen_info->sidewalk[2],
+                        0, -1,
+                        screen_info->width, 1, FALSE);
+
+        /*bottom*/
+        xfwmWindowShow (&screen_info->sidewalk[3],
+                        0, screen_info->height,
+                        screen_info->width, 1, FALSE);
     }
 }
 
-/* 
+/*
    gdk_screen_get_monitor_at_point () doesn't give accurate results
    when the point is off screen, use my own implementation from xfce 3
  */
-gint 
+gint
 find_monitor_at_point (GdkScreen *screen, gint x, gint y)
 {
     static gint cache_monitor = -1;
-    static gint cache_x;
-    static gint cache_y;
-
-    gint dx, dy;
-    gint center_x, center_y;
+    static gint cache_x, cache_y;
+    gint dx, dy, center_x, center_y;
     guint32 distsquare, min_distsquare;
-
-    gint num_monitors, i;
-    gint nearest_monitor;
+    gint num_monitors, nearest_monitor, i;
     GdkRectangle monitor;
 
     g_return_val_if_fail (GDK_IS_SCREEN (screen), -1);
@@ -209,7 +256,7 @@ find_monitor_at_point (GdkScreen *screen, gint x, gint y)
     {
         return (cache_monitor);
     }
-    
+
     cache_x = x;
     cache_y = y;
 
@@ -248,4 +295,3 @@ find_monitor_at_point (GdkScreen *screen, gint x, gint y)
     cache_monitor = nearest_monitor;
     return (nearest_monitor);
 }
-
