@@ -1,28 +1,34 @@
-/*      $Id: stacking.c 21716 2006-05-18 19:11:40Z olivier $
- 
+/*      $Id: stacking.c 27356 2008-07-20 21:22:15Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
- 
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
+
+        xfwm4    - (c) 2002-2008 Olivier Fourdan
+
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 #include <glib.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
 
 #include "display.h"
 #include "screen.h"
@@ -34,6 +40,8 @@
 #include "frame.h"
 #include "focus.h"
 
+static guint raise_timeout = 0;
+
 void
 clientApplyStackList (ScreenInfo *screen_info)
 {
@@ -154,28 +162,29 @@ gboolean
 clientIsTopMost (Client *c)
 {
     ScreenInfo *screen_info;
-    GList *index;
-    
+    GList *index, *index2;
+    Client *c2;
+
     g_return_val_if_fail (c != NULL, FALSE);
     TRACE ("entering clientIsTopMost");
-    
+
     screen_info = c->screen_info;
-    
+
     index = g_list_find (screen_info->windows_stack, (gconstpointer) c);
     if (index)
     {
-        GList *index2 = g_list_next (index);
-        if (index2)
+        index2 = g_list_next (index);
+        while (index2)
         {
-            Client *c2 = (Client *) index2->data;
-            return (c2->win_layer > c->win_layer);
-        }
-        else
-        {
-            return TRUE;
+            c2 = (Client *) index2->data;
+            if (FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_VISIBLE) && (c2->win_layer == c->win_layer))
+            {
+                return FALSE;
+            }
+            index2 = g_list_next (index2);
         }
     }
-    return FALSE;
+    return TRUE;
 }
 
 Client *
@@ -241,7 +250,7 @@ clientGetBottomMost (ScreenInfo *screen_info, int layer, Client * exclude)
  * conditions caused by querying the X server
  */
 Client *
-clientAtPosition (ScreenInfo *screen_info, int x, int y, Client * exclude)
+clientAtPosition (ScreenInfo *screen_info, int x, int y, GList * exclude_list)
 {
     GList *index;
     Client *c, *c2;
@@ -252,11 +261,11 @@ clientAtPosition (ScreenInfo *screen_info, int x, int y, Client * exclude)
     for (index = g_list_last (screen_info->windows_stack); index; index = g_list_previous (index))
     {
         c2 = (Client *) index->data;
-        if ((frameX (c2) < x) && (frameX (c2) + frameWidth (c2) > x)
-            && (frameY (c2) < y) && (frameY (c2) + frameHeight (c2) > y))
+        if ((frameX (c2) <= x) && (frameX (c2) + frameWidth (c2) >= x)
+            && (frameY (c2) <= y) && (frameY (c2) + frameHeight (c2) >= y))
         {
-            if (clientSelectMask (c2, 0, WINDOW_REGULAR_FOCUSABLE) 
-                && (c2 != exclude))
+            if (clientSelectMask (c2, INCLUDE_SKIP_PAGER | INCLUDE_SKIP_TASKBAR, WINDOW_REGULAR_FOCUSABLE)
+                && !g_list_find (exclude_list, (gconstpointer) c2))
             {
                 c = c2;
                 break;
@@ -295,11 +304,11 @@ clientRaise (Client * c, Window wsibling)
     }
     TRACE ("raising client \"%s\" (0x%lx) over (0x%lx)", c->name, c->window, wsibling);
 
-    /* 
+    /*
      * If the raised window is the one that has focus, fine, we can
      * release the grab we have on it since there is no use for it
      * anymore.
-     * 
+     *
      * However, if the raised window is not the focused one, then we
      * end up with some kind of indermination, so we need to regrab
      * the buttons for the user to be able to raise or focus the window
@@ -327,7 +336,7 @@ clientRaise (Client * c, Window wsibling)
         /* Search for the window that will be just on top of the raised window  */
         if (wsibling)
         {
-            c2 = myDisplayGetClientFromWindow (display_info, wsibling, ANY);
+            c2 = myDisplayGetClientFromWindow (display_info, wsibling, SEARCH_FRAME | SEARCH_WINDOW);
             if (c2)
             {
                 sibling = g_list_find (screen_info->windows_stack, (gconstpointer) c2);
@@ -474,7 +483,7 @@ clientLower (Client * c, Window wsibling)
         }
         else if (wsibling)
         {
-            c2 = myDisplayGetClientFromWindow (display_info, wsibling, ANY);
+            c2 = myDisplayGetClientFromWindow (display_info, wsibling, SEARCH_FRAME | SEARCH_WINDOW);
             if (c2)
             {
                 sibling = g_list_find (screen_info->windows_stack, (gconstpointer) c2);
@@ -493,7 +502,7 @@ clientLower (Client * c, Window wsibling)
                 }
             }
         }
-        if ((!client_sibling) || 
+        if ((!client_sibling) ||
             (client_sibling && (client_sibling->win_layer < c->win_layer)))
         {
             client_sibling = clientGetBottomMost (screen_info, c->win_layer, c);
@@ -540,11 +549,11 @@ clientAdjustFullscreenLayer (Client *c, gboolean set)
         if (FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN)
             || FLAG_TEST(c->flags, CLIENT_FLAG_FULLSCREEN))
         {
-            clientSetLayer (c, WIN_LAYER_ABOVE_DOCK);
+            clientSetLayer (c, WIN_LAYER_FULLSCREEN);
             return TRUE;
         }
     }
-    else if (c->win_layer == WIN_LAYER_ABOVE_DOCK)
+    else if (c->win_layer == WIN_LAYER_FULLSCREEN)
     {
         if (FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN)
             || FLAG_TEST(c->flags, CLIENT_FLAG_FULLSCREEN))
@@ -681,3 +690,44 @@ clientClearLastRaise (ScreenInfo *screen_info)
     g_return_if_fail (screen_info);
     screen_info->last_raise = NULL;
 }
+
+static gboolean
+delayed_raise_cb (gpointer data)
+{
+    Client *c;
+
+    TRACE ("entering delayed_raise_cb");
+
+    clientClearDelayedRaise ();
+    c = clientGetFocus ();
+
+    if (c)
+    {
+        clientRaise (c, None);
+    }
+    return (TRUE);
+}
+
+void
+clientClearDelayedRaise (void)
+{
+    if (raise_timeout)
+    {
+        g_source_remove (raise_timeout);
+        raise_timeout = 0;
+    }
+}
+
+void
+clientResetDelayedRaise (ScreenInfo *screen_info)
+{
+    if (raise_timeout)
+    {
+        g_source_remove (raise_timeout);
+    }
+    raise_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT,
+                                        screen_info->params->raise_delay,
+                                        (GSourceFunc) delayed_raise_cb,
+                                        NULL, NULL);
+}
+
